import * as bunyan from 'bunyan';
import { Match, InputData, FileService } from '@ethereum-sourcify/core';
export interface InjectorConfig {
    infuraPID?: string;
    localChainUrl?: string;
    silent?: boolean;
    log?: bunyan;
    offline?: boolean;
    repositoryPath?: string;
    fileService?: FileService;
}
export declare class Injector {
    private log;
    private chains;
    private infuraPID;
    private localChainUrl;
    private offline;
    fileService: FileService;
    repositoryPath: string;
    /**
     * Constructor
     * @param {InjectorConfig = {}} config
     */
    private constructor();
    /**
     * Creates an instance of Injector. Waits for chains to initialize.
     * Await this method to work with an instance that has all chains initialized.
     * @param config
     */
    static createAsync(config?: InjectorConfig): Promise<Injector>;
    /**
     * Instantiates a web3 provider for all public ethereum networks via Infura or regular node.
     * If environment variable TESTING is set to true, localhost:8545 is also available.
     */
    private initChains;
    /**
     * Searches a set of addresses for the one whose deployedBytecode
     * matches a given bytecode string
     * @param {String[]}          addresses
     * @param {string}      deployedBytecode
     */
    private matchBytecodeToAddress;
    /**
     * Returns a string description of how closely two bytecodes match. Bytecodes
     * that match in all respects apart from their metadata hashes are 'partial'.
     * Bytecodes that don't match are `null`.
     * @param  {string} deployedBytecode
     * @param  {string} compiledBytecode
     * @return {string | null}  match description ('perfect'|'partial'|null)
     */
    private compareBytecodes;
    /**
     * Throws if addresses array contains a null value (express) or is length 0
     * @param {string[] = []} addresses param (submitted to injector)
     */
    private validateAddresses;
    /**
     * Throws if `chain` is falsy or wrong type
     * @param {string} chain param (submitted to injector)
     */
    private validateChain;
    /**
     * Used by the front-end. Accepts a set of source files and a metadata string,
     * recompiles / validates them and stores them in the repository by chain/address
     * and by swarm | ipfs hash.
     * @param  {string}            repository repository root (ex: 'repository')
     * @param  {string}            chain      chain name (ex: 'ropsten')
     * @param  {string}            address    contract address
     * @param  {string[]}          files
     * @return {Promise<object>}              address & status of successfully verified contract
     */
    inject(inputData: InputData): Promise<Match>;
    /**
   * Writes verified sources to repository by address and by ipfs | swarm hash
   * @param {string}              repository        repository root (ex: 'repository')
   * @param {string}              chain             chain name (ex: 'ropsten')
   * @param {string}              address           contract address
   * @param {RecompilationResult} compilationResult solc output
   * @param {StringMap}           sources           'rearranged' sources
   */
    private storePerfectMatchData;
    /**
     * Writes verified sources to repository by address under the "partial_match" folder.
     * This method used when recompilation bytecode matches deployed *except* for their
     * metadata components.
     * @param {string}              repository        repository root (ex: 'repository')
     * @param {string}              chain             chain name (ex: 'ropsten')
     * @param {string}              address           contract address
     * @param {RecompilationResult} compilationResult solc output
     * @param {StringMap}           sources           'rearranged' sources
     */
    private storePartialMatchData;
}
