"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Injector = void 0;
const web3_1 = __importDefault(require("web3"));
const path_1 = __importDefault(require("path"));
const core_1 = require("@ethereum-sourcify/core");
const utils_1 = require("../utils");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const multihashes = require('multihashes');
class Injector {
    /**
     * Constructor
     * @param {InjectorConfig = {}} config
     */
    constructor(config = {}) {
        this.chains = {};
        this.infuraPID = config.infuraPID;
        this.localChainUrl = config.localChainUrl;
        this.offline = config.offline || false;
        this.repositoryPath = config.repositoryPath;
        this.log = config.log || core_1.Logger("Injector");
        this.fileService = config.fileService || new core_1.FileService(this.repositoryPath, this.log);
    }
    /**
     * Creates an instance of Injector. Waits for chains to initialize.
     * Await this method to work with an instance that has all chains initialized.
     * @param config
     */
    static async createAsync(config = {}) {
        const instance = new Injector(config);
        if (!instance.offline) {
            await instance.initChains();
        }
        return instance;
    }
    /**
     * Instantiates a web3 provider for all public ethereum networks via Infura or regular node.
     * If environment variable TESTING is set to true, localhost:8545 is also available.
     */
    async initChains() {
        if (this.infuraPID) {
            this.log.info({ loc: "[INIT_CHAINS]" }, "started checking infuraPID");
            await utils_1.checkEndpoint(this.infuraPID).catch((err) => {
                this.log.warn({ infuraID: this.infuraPID }, err.message);
            });
            this.log.info({ loc: "[INIT_CHAINS]" }, "finished checking infuraPID");
        }
        for (const chain of core_1.getSupportedChains()) {
            this.chains[chain.chainId] = {};
            if (this.infuraPID) {
                const web3 = chain.web3[0].replace('${INFURA_ID}', this.infuraPID);
                this.chains[chain.chainId].web3 = new web3_1.default(web3);
            }
            else {
                const web3 = chain.fullnode.dappnode;
                this.chains[chain.chainId].web3 = new web3_1.default(web3);
                await utils_1.checkEndpoint(web3).catch(() => {
                    this.log.warn({ endpoint: web3 }, `Invalid endpoint for chain ${chain.name}`);
                });
            }
        }
        const numberOfChains = Object.keys(this.chains).length;
        this.log.info({ loc: "[INIT_CHAINS]", numberOfChains }, "Finished loading chains");
        // For unit testing with testrpc...
        if (this.localChainUrl) {
            const chainOption = core_1.getChainByName('Localchain');
            this.chains[chainOption.chainId] = {
                web3: new web3_1.default(chainOption.web3[0])
            };
        }
    }
    /**
     * Searches a set of addresses for the one whose deployedBytecode
     * matches a given bytecode string
     * @param {String[]}          addresses
     * @param {string}      deployedBytecode
     */
    async matchBytecodeToAddress(chain, addresses = [], compiledBytecode) {
        let match = { address: null, status: null };
        for (let address of addresses) {
            address = web3_1.default.utils.toChecksumAddress(address);
            let deployedBytecode = null;
            try {
                this.log.info({
                    loc: '[MATCH]',
                    chain: chain,
                    address: address
                }, `Retrieving contract bytecode address`);
                deployedBytecode = await utils_1.getBytecode(this.chains[chain].web3, address);
            }
            catch (e) { /* ignore */ }
            const status = this.compareBytecodes(deployedBytecode, compiledBytecode);
            if (status) {
                match = { address: address, status: status };
                break;
            }
        }
        return match;
    }
    /**
     * Returns a string description of how closely two bytecodes match. Bytecodes
     * that match in all respects apart from their metadata hashes are 'partial'.
     * Bytecodes that don't match are `null`.
     * @param  {string} deployedBytecode
     * @param  {string} compiledBytecode
     * @return {string | null}  match description ('perfect'|'partial'|null)
     */
    compareBytecodes(deployedBytecode, compiledBytecode) {
        if (deployedBytecode && deployedBytecode.length > 2) {
            if (deployedBytecode === compiledBytecode) {
                return 'perfect';
            }
            if (utils_1.getBytecodeWithoutMetadata(deployedBytecode) === utils_1.getBytecodeWithoutMetadata(compiledBytecode)) {
                return 'partial';
            }
        }
        return null;
    }
    /**
     * Throws if addresses array contains a null value (express) or is length 0
     * @param {string[] = []} addresses param (submitted to injector)
     */
    validateAddresses(addresses = []) {
        const err = new Error("Missing address for submitted sources/metadata");
        if (!addresses.length) {
            throw err;
        }
        for (const address of addresses) {
            if (address == null)
                throw err;
        }
    }
    /**
     * Throws if `chain` is falsy or wrong type
     * @param {string} chain param (submitted to injector)
     */
    validateChain(chain) {
        if (!chain || typeof chain !== 'string') {
            throw new Error("Missing chain name for submitted sources/metadata");
        }
    }
    /**
     * Used by the front-end. Accepts a set of source files and a metadata string,
     * recompiles / validates them and stores them in the repository by chain/address
     * and by swarm | ipfs hash.
     * @param  {string}            repository repository root (ex: 'repository')
     * @param  {string}            chain      chain name (ex: 'ropsten')
     * @param  {string}            address    contract address
     * @param  {string[]}          files
     * @return {Promise<object>}              address & status of successfully verified contract
     */
    async inject(inputData) {
        const { chain, addresses, files } = inputData;
        this.validateAddresses(addresses);
        this.validateChain(chain);
        let match = {
            address: null,
            status: null
        };
        for (const source of files) {
            // Starting from here, we cannot trust the metadata object anymore,
            // because it is modified inside recompile.
            const target = Object.assign({}, source.metadata.settings.compilationTarget);
            let compilationResult;
            try {
                compilationResult = await utils_1.recompile(source.metadata, source.solidity, this.log);
            }
            catch (err) {
                this.log.info({ loc: `[RECOMPILE]`, err: err });
                throw err;
            }
            // When injector is called by monitor, the bytecode has already been
            // obtained for address and we only need to compare w/ compilation result.
            if (inputData.bytecode) {
                const status = this.compareBytecodes(inputData.bytecode, compilationResult.deployedBytecode);
                match = {
                    address: web3_1.default.utils.toChecksumAddress(addresses[0]),
                    status: status
                };
                // For other cases, we need to retrieve the code for specified address
                // from the chain.
            }
            else {
                match = await this.matchBytecodeToAddress(chain, addresses, compilationResult.deployedBytecode);
            }
            // Since the bytecode matches, we can be sure that we got the right
            // metadata file (up to json formatting) and exactly the right sources.
            // Now we can store the re-compiled and correctly formatted metadata file
            // and the sources.
            if (match.address && match.status === 'perfect') {
                this.storePerfectMatchData(this.repositoryPath, chain, match.address, compilationResult, source.solidity);
            }
            else if (match.address && match.status === 'partial') {
                this.storePartialMatchData(this.repositoryPath, chain, match.address, compilationResult, source.solidity);
            }
            else {
                const err = new Error(`Could not match on-chain deployed bytecode to recompiled bytecode for:\n` +
                    `${JSON.stringify(target, null, ' ')}\n` +
                    `Addresses checked:\n` +
                    `${JSON.stringify(addresses, null, ' ')}`);
                this.log.info({
                    loc: '[INJECT]',
                    chain: chain,
                    addresses: addresses,
                    err: err
                });
                throw new Error(err.message);
            }
        }
        return match;
    }
    /**
   * Writes verified sources to repository by address and by ipfs | swarm hash
   * @param {string}              repository        repository root (ex: 'repository')
   * @param {string}              chain             chain name (ex: 'ropsten')
   * @param {string}              address           contract address
   * @param {RecompilationResult} compilationResult solc output
   * @param {StringMap}           sources           'rearranged' sources
   */
    storePerfectMatchData(repository, chain, address, compilationResult, sources) {
        let metadataPath;
        const bytes = web3_1.default.utils.hexToBytes(compilationResult.deployedBytecode);
        const cborData = core_1.cborDecode(bytes);
        if (cborData['bzzr0']) {
            metadataPath = `/swarm/bzzr0/${web3_1.default.utils.bytesToHex(cborData['bzzr0']).slice(2)}`;
        }
        else if (cborData['bzzr1']) {
            metadataPath = `/swarm/bzzr1/${web3_1.default.utils.bytesToHex(cborData['bzzr1']).slice(2)}`;
        }
        else if (cborData['ipfs']) {
            metadataPath = `/ipfs/${multihashes.toB58String(cborData['ipfs'])}`;
        }
        else {
            const err = new Error("Re-compilation successful, but could not find reference to metadata file in cbor data.");
            this.log.info({
                loc: '[STOREDATA]',
                address: address,
                chain: chain,
                err: err
            });
            throw err;
        }
        const hashPath = path_1.default.join(repository, metadataPath);
        const addressPath = path_1.default.join(repository, 'contracts', 'full_match', chain, address, '/metadata.json');
        this.fileService.save(hashPath, compilationResult.metadata);
        this.fileService.save(addressPath, compilationResult.metadata);
        for (const sourcePath in sources) {
            const sanitizedPath = sourcePath
                .replace(/[^a-z0-9_./-]/gim, "_")
                .replace(/(^|\/)[.]+($|\/)/, '_');
            const outputPath = path_1.default.join(repository, 'contracts', 'full_match', chain, address, 'sources', sanitizedPath);
            this.fileService.save(outputPath, sources[sourcePath]);
        }
    }
    /**
     * Writes verified sources to repository by address under the "partial_match" folder.
     * This method used when recompilation bytecode matches deployed *except* for their
     * metadata components.
     * @param {string}              repository        repository root (ex: 'repository')
     * @param {string}              chain             chain name (ex: 'ropsten')
     * @param {string}              address           contract address
     * @param {RecompilationResult} compilationResult solc output
     * @param {StringMap}           sources           'rearranged' sources
     */
    storePartialMatchData(repository, chain, address, compilationResult, sources) {
        const addressPath = path_1.default.join(repository, 'contracts', 'partial_match', chain, address, '/metadata.json');
        this.fileService.save(addressPath, compilationResult.metadata);
        for (const sourcePath in sources) {
            const sanitizedPath = sourcePath
                .replace(/[^a-z0-9_./-]/gim, "_")
                .replace(/(^|\/)[.]+($|\/)/, '_');
            const outputPath = path_1.default.join(repository, 'contracts', 'partial_match', chain, address, 'sources', sanitizedPath);
            this.fileService.save(outputPath, sources[sourcePath]);
        }
    }
}
exports.Injector = Injector;
//# sourceMappingURL=Injector.js.map